

# plotMad
# -----------------------------------------------------------------------------
# 24/04/2014

plotMad <- function(dt, chr, main="", save.as=NA, legend=FALSE, bp.from=NA, 
    bp.to=NA, rt=FALSE, pch=20, pch.cex=2, width=par("din")[1], 
    height=par("din")[2]) {
    
    if(class(dt) == "character") {l
        dt <- read.delim(dt, header=TRUE, sep="\t")
    }


    if (!("Chr" %in% colnames(dt) & "Position" %in% colnames(dt) & "GType" %in% colnames(dt) &
        "Log.R.Ratio" %in% colnames(dt) & "B.Allele.Freq" %in% colnames(dt))) {
        stop("The input 'data.frame' must contain columns 'Chr', 'Position', 'Log.R.Ratio', 'B.Allele.Freq' and 'GType'.")
    }

    color.baf <- c("red", "salmon", "red")
    names(color.baf) <- c("AA", "AB", "BB")

    # filter data
    dt <- dt[dt$Chr == chr, ]
    dt <- dt[order(dt$Position), ]
    dt$Log.R.Ratio[dt$Log.R.Ratio < -1] <- -1
    dt$Log.R.Ratio[dt$Log.R.Ratio > 1] <- 1

    if (!is.na(bp.from)) {
        dt <- dt[dt$Position >= bp.from, ]
    }
    if (!is.na(bp.to)) {
        dt <- dt[dt$Position <= bp.to, ]
    }
    # /

    if (!rt) {
        grid.newpage()
    }

    # create LRR and BAF plots
    plot.lrr <- ggplot(data = dt, aes(x = Position, y = Log.R.Ratio, color="LRR")) 
    plot.lrr <- plot.lrr + geom_point(shape = pch, size=pch.cex) + ggtitle(main)
    plot.lrr <- plot.lrr + theme(plot.title = element_text(lineheight = 0.8, face = "bold"))
    if (legend) {
        plot.lrr <- plot.lrr + scale_color_manual(name = "", labels = c("LRR"), values = c("black"))
        plot.lrr <- plot.lrr + theme(legend.position = "bottom")#, legend.direction = "horizontal", legend.margin=unit(c(0, 0, 0, 0), "cm"))
    } else {
        plot.lrr <- plot.lrr + scale_color_manual(name = "", labels = c("LRR"), values = c("black"), guide = FALSE)
    }

    plot.baf <- ggplot(data = dt, aes(x = Position, y = B.Allele.Freq, color=GType))
    plot.baf <- plot.baf + geom_point(shape = pch, size=pch.cex)
    plot.baf <- plot.baf + theme(panel.background = element_rect(fill = NA), panel.grid = element_blank())
    plot.baf <- plot.baf + scale_color_manual(name = "GType", labels = c("AA", "AB", "BB"), values = color.baf)
    if (legend) {
        plot.baf <- plot.baf + theme(legend.position = "bottom")#, legend.direction = "horizontal", legend.margin=unit(c(-5,-5,-5,-5), "cm"))
    }
    # /

    # extract gtable from each plot
    gtab.lrr <- ggplot_gtable(ggplot_build(plot.lrr))
    gtab.baf <- ggplot_gtable(ggplot_build(plot.baf))
    # /

    # overlap the panel of BAF plot on the one of LRR plot
    pp <- c(subset(gtab.lrr$layout, name == "panel", se = t:r))
    plot <- gtable_add_grob(gtab.lrr, gtab.baf$grobs[[which(gtab.baf$layout$name == "panel")]], pp$t, 
        pp$l, pp$b, pp$l)
    # /

    # mixing axis
    ia <- which(gtab.baf$layout$name == "axis-l")
    ga <- gtab.baf$grobs[[ia]]
    ax <- ga$children[[2]]
    ax$widths <- rev(ax$widths)
    ax$grobs <- rev(ax$grobs)
    ax$grobs[[1]]$x <- ax$grobs[[1]]$x - unit(1, "npc") + unit(0.15, "cm")
    plot <- gtable_add_cols(plot, gtab.baf$widths[gtab.baf$layout[ia, ]$l], length(plot$widths) - 1)
    plot <- gtable_add_grob(plot, ax, pp$t, length(plot$widths) - 1, pp$b)
    # /

    # second label for y axis (BAF)
    ia <- which(gtab.baf$layout$name == "ylab")
    ylab <- gtab.baf$grobs[[ia]]
    plot <- gtable_add_cols(plot, plot$widths[plot$layout[ia, ]$l], length(plot$widths) - 1)
    plot <- gtable_add_grob(plot, ylab, pp$t, length(plot$widths) - 1, pp$b)
    # /

    # seconds legend for y axis (BAF)
    if (legend) {
        ia <- which(gtab.baf$layout$name == "guide-box")
        #yleg <- gtab.baf$grobs[[ia]]
        plot <- gtable_add_rows(plot, gtab.baf$heights[gtab.baf$layout[ia, ]$b] - unit(0.15, "cm"), length(gtab.baf$heights) - 1)
        #plot <- gtable_add_grob(plot, yleg, length(plot$heights) - 1, pp$l, pp$b)
    }
    # /

    if (rt) {
        return(plot)
    }

    if (!is.na(save.as)) {
        ggsave(filename = save.as, plot = plot, width = width, height = height)
    } else {
        grid.draw(plot)
    }
}